#include "me_xbm.h"

struct STempVertex
{
	float x,y,z;
	float u, v;
} ;

me_CXBM::me_CXBM()
{
	nObject=0;
}

me_CXBM::~me_CXBM()
{
	int i;
	for(i = 0; i < nObject; i++)
	{		
		me_SXBMObject *tObject=&Object[i];	
		
		if(tObject->nVertex==0) continue;
		
		delete [] tObject->Vertex;
		delete [] tObject->Face;
		delete [] tObject->Texcoord;
		
		me_DeleteTexture(tObject->Texture);
	}
	
	delete [] Object;
}

void me_CXBM::Render(int cur_frame)
{	
	int i;
	
	cur_frame %= nFrame;
	
	me_SXBMObject *tObject;
	for(i = 0; i < nObject; i++)
	{	
		tObject=&Object[i];
		if(tObject->nVertex==0) continue;

		int currentIndex = cur_frame * tObject->nVertex * 3; 

		glBindTexture(GL_TEXTURE_2D, tObject->Texture.TexID);
		
		glVertexPointer(3, GL_FLOAT, 0, tObject->Vertex+currentIndex);
		glEnableClientState(GL_VERTEX_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, 0, tObject->Texcoord);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		
		glDrawElements(GL_TRIANGLES, tObject->nFace, GL_UNSIGNED_SHORT, tObject->Face);
	}
}

void me_CXBM::Load(const char *filename)
{
	const char *path = me_GetBundlePath(filename, 0);
	FILE *fp;
	fp = fopen(path, "rb");
	if(fp==NULL)
	{
		NSLog(@"me_CXBM::load fail.");
		return;
	}
	
	fread(Header.ID, 4, 1, fp);
	fread(&Header.Version, 1, 1, fp);
	
	if(strncmp(Header.ID, "XBM", 3)) 
	{
		NSLog(@"me_CXBM::Wrong header.");
		fclose(fp);
		return;
	}

	fclose(fp);

	LoadObject(path);	

	int i;
	for(i=0;i<nObject;i++)
	{
		if(Object[i].nVertex==0) continue;
		if(Object[i].TexName[0]==0) continue;
				
		Object[i].Texture=me_CreateTexture([[NSString alloc] initWithCString:Object[i].TexName]);		
	}
}

void me_CXBM::LoadObject(const char* filename)	
{	
	int nVertex;
	me_SXBMVertex *Vertex;
	int nTexcoord;
	me_SXBMTexcoord *Texcoord;

	int nFace;
	me_SXBMFace *Face;
	me_SXBMFace *TexFace;	
	
	STempVertex *tempVertexBuffer;
	STempVertex *tempOptVertexBuffer;
	unsigned char *tempVertexSkipInfo;
	
	FILE *fp;
	fp = fopen(filename, "rb");
	if(fp==NULL)
	{
		NSLog(@"loading fail.");
		exit(1);
	}

	fread(Header.ID, 4, 1, fp);
	fread(&Header.Version, 1, 1, fp);

	fread(&nObject, 1, 1, fp);
	Object=new me_SXBMObject [ nObject ];

	fread(&nFrame, 4, 1, fp);
	nFrame++;

	int i,j,k;
	for(i=0;i<nObject;i++)
	{			
		me_SXBMObject *tObject=&Object[i];

		k=0;
		while(1)
		{
			char c;
			c = fgetc(fp);
			tObject->Name[k]=c;
			if(tObject->Name[k]==0) break;
			k++;
		}

		k=0;
		while(1)
		{
			char c;
			c = fgetc(fp);
			tObject->TexName[k]=c;
			if(tObject->TexName[k]==0) break;
			k++;
		}

		/////////////////////////////////////////////////////////////////////////
		fread(&nVertex, 4,  1, fp);
		Vertex=new me_SXBMVertex [nVertex*nFrame ];	
			
		for(j=0;j<nVertex*nFrame;j++)	
		{
			fread(&Vertex[j].x, 4, 1, fp);				
			fread(&Vertex[j].y, 4, 1, fp);				
			fread(&Vertex[j].z, 4, 1, fp);			
		}

		fread(&nTexcoord, 4, 1, fp);
		Texcoord=new me_SXBMTexcoord [ nTexcoord ];
		for(j=0;j<nTexcoord;j++)	
		{
			fread(&Texcoord[j].u, 4, 1, fp);				
			fread(&Texcoord[j].v, 4, 1, fp);
		}

		fread(&nFace, 4, 1, fp);
		Face=new me_SXBMFace [ nFace ];		
		for(j=0;j<nFace;j++)	
		{
			short d;

			fread(&d, 2, 1, fp);			Face[j].p1=d-1;
			fread(&d, 2, 1, fp);			Face[j].p3=d-1;
			fread(&d, 2, 1, fp);			Face[j].p2=d-1;
		}

		TexFace=new me_SXBMFace [ nFace ];		
		for(j=0;j<nFace;j++)	
		{
			short d;

			fread(&d, 2, 1, fp);			TexFace[j].p1=d-1;
			fread(&d, 2, 1, fp);			TexFace[j].p3=d-1;
			fread(&d, 2, 1, fp);			TexFace[j].p2=d-1;
		}	

		/////////////////////////////////////////////////////////////////////////
		tObject->nVertex=0;			
		tObject->Vertex=NULL;
		tObject->Texcoord=NULL;
		tObject->Face=NULL;

		tempVertexBuffer=new STempVertex [ nFace*4 ];
		tempOptVertexBuffer= new STempVertex [ nFace*4 ];
		tempVertexSkipInfo= new unsigned char [ nFace*4 ];

		int tindex, index=0;
		for(j = 0; j < nFace; j++)
		{
			tindex=Face[j].p1;
			tempVertexBuffer[index].x=Vertex[tindex].x;
			tempVertexBuffer[index].y=Vertex[tindex].y;
			tempVertexBuffer[index].z=Vertex[tindex].z;
			tindex=TexFace[j].p1;
			tempVertexBuffer[index].u=Texcoord[tindex].u;
			tempVertexBuffer[index].v=Texcoord[tindex].v;
			index++;

			tindex=Face[j].p2;
			tempVertexBuffer[index].x=Vertex[tindex].x;
			tempVertexBuffer[index].y=Vertex[tindex].y;
			tempVertexBuffer[index].z=Vertex[tindex].z;
			tindex=TexFace[j].p2;
			tempVertexBuffer[index].u=Texcoord[tindex].u;
			tempVertexBuffer[index].v=Texcoord[tindex].v;
			index++;

			tindex=Face[j].p3;
			tempVertexBuffer[index].x=Vertex[tindex].x;
			tempVertexBuffer[index].y=Vertex[tindex].y;
			tempVertexBuffer[index].z=Vertex[tindex].z;
			tindex=TexFace[j].p3;
			tempVertexBuffer[index].u=Texcoord[tindex].u;
			tempVertexBuffer[index].v=Texcoord[tindex].v;
			index++;
		}

		int a, b, c=0;
		for(a=0;a<nFace*3;a++)
		{
			tempVertexSkipInfo[a]=0;
			int Flag=0;
			for(b=0;b<c;b++)
			{
				if( tempVertexBuffer[a].x==tempOptVertexBuffer[b].x &&
					tempVertexBuffer[a].y==tempOptVertexBuffer[b].y &&
					tempVertexBuffer[a].z==tempOptVertexBuffer[b].z &&
					tempVertexBuffer[a].u==tempOptVertexBuffer[b].u &&
					tempVertexBuffer[a].v==tempOptVertexBuffer[b].v)
				{
					Flag=1;
					break;
				}
			}

			if(Flag) continue;
			tempVertexSkipInfo[a]=1;
			tempOptVertexBuffer[c++]=tempVertexBuffer[a];	
		}

		tObject->nVertex=c;
		tObject->nFace=nFace*3;
		tObject->Face=new unsigned short [ nFace*3 ];	

		for(a=0;a<nFace*3;a++)
		{
			for(b=0;b<tObject->nVertex;b++)
			{
				if( tempVertexBuffer[a].x==tempOptVertexBuffer[b].x &&
					tempVertexBuffer[a].y==tempOptVertexBuffer[b].y &&
					tempVertexBuffer[a].z==tempOptVertexBuffer[b].z &&
					tempVertexBuffer[a].u==tempOptVertexBuffer[b].u &&
					tempVertexBuffer[a].v==tempOptVertexBuffer[b].v)	
				{
					tObject->Face[a]=b;					
					break;
				}
			}
		}

		tObject->Texcoord=new float [tObject->nVertex*2];
		for(j=0;j<tObject->nVertex;j++)	
		{
			tObject->Texcoord[j*2  ]=tempOptVertexBuffer[j].u;
			tObject->Texcoord[j*2+1]=1.0f-tempOptVertexBuffer[j].v;
		}

		tObject->Vertex= new float [tObject->nVertex*3*nFrame];
		for(k=0;k<nFrame;k++)
		{		
			index=0;
			for(j = 0; j < nFace; j++)
			{
				if(tempVertexSkipInfo[j*3  ])
				{
					tindex=Face[j].p1;
					tempOptVertexBuffer[index].x=Vertex[tindex+k*nVertex].x;
					tempOptVertexBuffer[index].y=Vertex[tindex+k*nVertex].y;
					tempOptVertexBuffer[index].z=Vertex[tindex+k*nVertex].z;
					index++;
				}

				if(tempVertexSkipInfo[j*3+1])
				{
					tindex=Face[j].p2;
					tempOptVertexBuffer[index].x=Vertex[tindex+k*nVertex].x;
					tempOptVertexBuffer[index].y=Vertex[tindex+k*nVertex].y;
					tempOptVertexBuffer[index].z=Vertex[tindex+k*nVertex].z;
					index++;
				}

				if(tempVertexSkipInfo[j*3+2])
				{
					tindex=Face[j].p3;
					tempOptVertexBuffer[index].x=Vertex[tindex+k*nVertex].x;
					tempOptVertexBuffer[index].y=Vertex[tindex+k*nVertex].y;
					tempOptVertexBuffer[index].z=Vertex[tindex+k*nVertex].z;
					index++;
				}
			}			

			for(j=0;j<tObject->nVertex;j++)
			{
				tObject->Vertex[(k*tObject->nVertex+j)*3  ]= tempOptVertexBuffer[j].x;
				tObject->Vertex[(k*tObject->nVertex+j)*3+1]= tempOptVertexBuffer[j].y;
				tObject->Vertex[(k*tObject->nVertex+j)*3+2]= tempOptVertexBuffer[j].z;
			}
		}

		delete [] Vertex;
		delete [] Texcoord;
		delete [] Face;
		delete [] TexFace;
		
		delete [] tempVertexBuffer;
		delete [] tempOptVertexBuffer;
		delete [] tempVertexSkipInfo;
	}

	fclose(fp);	
}