//
//  Box2DViewController.m
//  Box2D
//
//  Created by 영호 천 on 10. 12. 11..
//  Copyright 2010 .. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>

#import "Box2DViewController.h"
#import "EAGLView.h"

#import "Box2D.h"

b2Vec2 gravity(0.0f, 98.0f);	
bool doSleep = true;
b2World g_World(gravity, doSleep);
b2Body *g_GroundBody, *g_DynamicBody;

class me_CContactListener : public b2ContactListener
{
public:
	void BeginContact(b2Contact* contact)
	{ 
		NSLog(@"BeginContact");
	}
		
	void EndContact(b2Contact* contact)
	{
		NSLog(@"EndContact");
	}
			
	void PreSolve(b2Contact* contact, const b2Manifold* oldManifold)
	{
		NSLog(@"PreSolve");
	}
	
	void PostSolve(b2Contact* contact, const b2ContactImpulse* impulse)
	{ 
		b2Body *body = contact->GetFixtureA()->GetBody();
		if(body->GetType() == b2_dynamicBody)
		{
			body->SetLinearVelocity(b2Vec2(0, -100000.0f));
		}
		
		body = contact->GetFixtureB()->GetBody();
		if(body->GetType() == b2_dynamicBody)
		{
			body->SetLinearVelocity(b2Vec2(0, -100000.0f));
		}
		
		NSLog(@"PostSolve");
	}
					
};
				
me_CContactListener contactListener;

@interface Box2DViewController ()
@property (nonatomic, retain) EAGLContext *context;@end

@implementation Box2DViewController

@synthesize animating, context;

- (void)awakeFromNib
{
    EAGLContext *aContext = nil;//[[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
    
    if (!aContext)
    {
        aContext = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
    }
    
    if (!aContext)
        NSLog(@"Failed to create ES context");
    else if (![EAGLContext setCurrentContext:aContext])
        NSLog(@"Failed to set ES context current");
    
	self.context = aContext;
	[aContext release];
	
    [(EAGLView *)self.view setContext:context];
    [(EAGLView *)self.view setFramebuffer];
    
    animating = FALSE;
    displayLinkSupported = FALSE;
    animationFrameInterval = 1;
    displayLink = nil;
    animationTimer = nil;
    
    // Use of CADisplayLink requires iOS version 3.1 or greater.
	// The NSTimer object is used as fallback when it isn't available.
    NSString *reqSysVer = @"3.1";
    NSString *currSysVer = [[UIDevice currentDevice] systemVersion];
    if ([currSysVer compare:reqSysVer options:NSNumericSearch] != NSOrderedAscending)
        displayLinkSupported = TRUE;
	
	b2BodyDef groundBodyDef;
	groundBodyDef.position.Set(160.0f, 400.0f);
	g_GroundBody = g_World.CreateBody(&groundBodyDef);
	b2PolygonShape groundBox;
	groundBox.SetAsBox(160.0f, 10.0f);
	b2FixtureDef fixtureDef;
	fixtureDef.shape = &groundBox;
	g_GroundBody->CreateFixture(&fixtureDef);
	
	
	b2BodyDef bodyDef;
	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(160.0f, 100.0f);	
	g_DynamicBody = g_World.CreateBody(&bodyDef);
	b2PolygonShape dynamicBox;	
	fixtureDef.shape = &dynamicBox;
	fixtureDef.density = 1.0f;
	fixtureDef.friction = 0.3f;
	dynamicBox.SetAsBox(10.0f, 10.0f);
	g_DynamicBody->CreateFixture(&fixtureDef);	
	
	g_World.SetContactListener(&contactListener);
}

- (void)dealloc
{
    if (program)
    {
        glDeleteProgram(program);
        program = 0;
    }
    
    // Tear down context.
    if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
    
    [context release];
    
    [super dealloc];
}

- (void)viewWillAppear:(BOOL)animated
{
    [self startAnimation];
    
    [super viewWillAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [self stopAnimation];
    
    [super viewWillDisappear:animated];
}

- (void)viewDidUnload
{
	[super viewDidUnload];
	
    if (program)
    {
        glDeleteProgram(program);
        program = 0;
    }

    // Tear down context.
    if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
	self.context = nil;	
}

- (NSInteger)animationFrameInterval
{
    return animationFrameInterval;
}

- (void)setAnimationFrameInterval:(NSInteger)frameInterval
{
    /*
	 Frame interval defines how many display frames must pass between each time the display link fires.
	 The display link will only fire 30 times a second when the frame internal is two on a display that refreshes 60 times a second. The default frame interval setting of one will fire 60 times a second when the display refreshes at 60 times a second. A frame interval setting of less than one results in undefined behavior.
	 */
    if (frameInterval >= 1)
    {
        animationFrameInterval = frameInterval;
        
        if (animating)
        {
            [self stopAnimation];
            [self startAnimation];
        }
    }
}

- (void)startAnimation
{
    if (!animating)
    {
        if (displayLinkSupported)
        {
            /*
			 CADisplayLink is API new in iOS 3.1. Compiling against earlier versions will result in a warning, but can be dismissed if the system version runtime check for CADisplayLink exists in -awakeFromNib. The runtime check ensures this code will not be called in system versions earlier than 3.1.
            */
            displayLink = [NSClassFromString(@"CADisplayLink") displayLinkWithTarget:self selector:@selector(drawFrame)];
            [displayLink setFrameInterval:animationFrameInterval];
            
            // The run loop will retain the display link on add.
            [displayLink addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
        }
        else
            animationTimer = [NSTimer scheduledTimerWithTimeInterval:(NSTimeInterval)((1.0 / 60.0) * animationFrameInterval) target:self selector:@selector(drawFrame) userInfo:nil repeats:TRUE];
        
        animating = TRUE;
    }
}

- (void)stopAnimation
{
    if (animating)
    {
        if (displayLinkSupported)
        {
            [displayLink invalidate];
            displayLink = nil;
        }
        else
        {
            [animationTimer invalidate];
            animationTimer = nil;
        }
        
        animating = FALSE;
    }
}

void DrawBox(float x, float y, float angle, float hx, float hy)
{
	GLfloat vertex[8] =
	{
		-hx,	-hy,
		hx,		-hy,
		-hx,	hy,
		hx,		hy
	};
	
	glPushMatrix();
	glTranslatef(x, y, 0);
	glRotatef(angle, 0, 0, 1);
	
	glVertexPointer(2, GL_FLOAT, 0, vertex);
	glEnableClientState(GL_VERTEX_ARRAY);
	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);	
	
	glPopMatrix();
}

- (void)drawFrame
{
    [(EAGLView *)self.view setFramebuffer];
    
    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    
	g_World.Step(1.0f/60.0f, 10, 10);
    g_World.ClearForces();
    b2Vec2 position = g_DynamicBody->GetPosition();
    float32 angle = g_DynamicBody->GetAngle();
	DrawBox(position.x, position.y, angle, 10, 10);
	
	position = g_GroundBody->GetPosition();
    angle = g_GroundBody->GetAngle();
	
	DrawBox(position.x, position.y, angle, 160, 10);
	
	/*for(b2Contact *c=g_World.GetContactList(); c!=NULL; c=c->GetNext())
	{
		b2Manifold *manifold;
		manifold=c->GetManifold();
		
		b2WorldManifold worldmanifold;
		c->GetWorldManifold(&worldmanifold);
		
		for(int i=0;i<manifold->pointCount;i++)
			NSLog(@"%f %f",worldmanifold.points[i].x, worldmanifold.points[i].y);
		
		c->SetEnabled(false);
	}*/
       
    [(EAGLView *)self.view presentFramebuffer];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc. that aren't in use.
}

@end
