//
//  Box2DViewController.m
//  Box2D
//
//  Created by 영호 천 on 10. 12. 11..
//  Copyright 2010 .. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>

#import "Box2DViewController.h"
#import "EAGLView.h"

#import "Box2D.h"

b2Vec2 gravity(0.0f, 98.0f);	
bool doSleep = true;
b2World g_World(gravity, doSleep);
b2Body *g_GroundBody, *g_Body[2];
b2Joint *g_Joint;


void DrawBox(float x, float y, float angle, float hx, float hy)
{
	GLfloat vertex[8] =
	{
		-hx,	-hy,
		hx,		-hy,
		-hx,	hy,
		hx,		hy
	};
	
	glPushMatrix();
	glTranslatef(x, y, 0);
	glRotatef(angle, 0, 0, 1);
	
	glVertexPointer(2, GL_FLOAT, 0, vertex);
	glEnableClientState(GL_VERTEX_ARRAY);
	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);	
	
	glPopMatrix();
}


void CreateDistanceJoint()
{
	b2BodyDef bodyDef;
	b2PolygonShape box;	
	box.SetAsBox(10.0f, 10.0f);
	
	b2FixtureDef fixtureDef;
	fixtureDef.shape = &box;
	
	bodyDef.type = b2_staticBody;
	bodyDef.position.Set(160.0f, 80.0f);	
	g_Body[0] = g_World.CreateBody(&bodyDef);
	g_Body[0]->CreateFixture(&fixtureDef);
	
	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(80.0f, 0.0f);	
	g_Body[1] = g_World.CreateBody(&bodyDef);
	g_Body[1]->CreateFixture(&fixtureDef);
	
	b2DistanceJointDef jointDef;
	jointDef.Initialize(g_Body[0], g_Body[1], b2Vec2(160, 80), b2Vec2(80, 0));
	jointDef.collideConnected = true;
	jointDef.frequencyHz = 4.0f;
	jointDef.dampingRatio = 0.1f;
	
	g_Joint = g_World.CreateJoint(&jointDef);
}

void RenderDistanceJoint()
{
	int i;
	for(i=0;i<2;i++)
	{
		b2Vec2 position = g_Body[i]->GetPosition();
		float32 angle = g_Body[i]->GetAngle();
		DrawBox(position.x, position.y, angle*57.29577951f, 10, 10);
	}
}

void CreateRevoluteJoint()
{
	b2BodyDef bodyDef;
	b2PolygonShape box1, box2;	
	box1.SetAsBox(10.0f, 50.0f);
	box2.SetAsBox(50.0f, 10.0f);
	
	b2FixtureDef fixtureDef;
	fixtureDef.density = 1.0f;
	
	fixtureDef.shape = &box1;	
	bodyDef.type = b2_staticBody;
	bodyDef.position.Set(160.0f, 80.0f);	
	g_Body[0] = g_World.CreateBody(&bodyDef);
	g_Body[0]->CreateFixture(&fixtureDef);
	
	fixtureDef.shape = &box2;
	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(200.0f, 130.0f);	
	g_Body[1] = g_World.CreateBody(&bodyDef);
	g_Body[1]->CreateFixture(&fixtureDef);
	
	b2RevoluteJointDef jointDef;
	jointDef.Initialize(g_Body[0], g_Body[1], b2Vec2(160.0f, 130.0f));
	jointDef.collideConnected = false;
	//jointDef.upperAngle = 0.5f * b2_pi;
	//jointDef.lowerAngle = -0.5f * b2_pi;
	//jointDef.enableLimit = true;
	//jointDef.maxMotorTorque = 10.0f;
	//jointDef.motorSpeed = 0.0f;
	//jointDef.enableMotor = true;
	
	g_Joint = g_World.CreateJoint(&jointDef);
}

void RenderRevoluteJoint()
{
	b2Vec2 position = g_Body[0]->GetPosition();
	float32 angle = g_Body[0]->GetAngle();
	DrawBox(position.x, position.y, angle*57.29577951f, 10, 50);
	
	position = g_Body[1]->GetPosition();
	angle = g_Body[1]->GetAngle();
	DrawBox(position.x, position.y, angle*57.29577951f, 50, 10);
}

void CreatePrismaticJoint()
{
	b2BodyDef bodyDef;
	b2PolygonShape box1, box2;	
	box1.SetAsBox(10.0f, 50.0f);
	box2.SetAsBox(50.0f, 10.0f);
	
	b2FixtureDef fixtureDef;
	fixtureDef.density = 1.0f;
	
	fixtureDef.shape = &box1;	
	bodyDef.type = b2_staticBody;
	bodyDef.position.Set(160.0f, 80.0f);	
	g_Body[0] = g_World.CreateBody(&bodyDef);
	g_Body[0]->CreateFixture(&fixtureDef);
	
	fixtureDef.shape = &box2;
	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(160.0f, 80.0f);
	bodyDef.angle = 0.25f * b2_pi;
	g_Body[1] = g_World.CreateBody(&bodyDef);
	g_Body[1]->CreateFixture(&fixtureDef);
	
	b2PrismaticJointDef jointDef;
	jointDef.Initialize(g_Body[0], g_Body[1], b2Vec2(160.0f, 80.0f), b2Vec2(0.0f, 1.0f));
	jointDef.lowerTranslation = -200.0f;
	jointDef.upperTranslation = 200.0f;
	jointDef.enableLimit = true;
	jointDef.maxMotorForce = 100.0f;
	jointDef.motorSpeed = 100.0f;
	jointDef.enableMotor = true;
	jointDef.collideConnected = false;
	
	g_Joint = g_World.CreateJoint(&jointDef);
}

void RenderPrismaticJoint()
{
	b2Vec2 position = g_Body[0]->GetPosition();
	float32 angle = g_Body[0]->GetAngle();
	DrawBox(position.x, position.y, angle*57.29577951f, 10, 50);
	
	position = g_Body[1]->GetPosition();
	angle = g_Body[1]->GetAngle();
	DrawBox(position.x, position.y, angle*57.29577951f, 50, 10);
}


void CreatePulleyJoint()
{
	b2BodyDef bodyDef;
	b2PolygonShape box1, box2;	
	box1.SetAsBox(20.0f, 20.0f);
	box2.SetAsBox(30.0f, 30.0f);
	
	b2FixtureDef fixtureDef;
	fixtureDef.density = 1.0f;
	
	fixtureDef.shape = &box1;	
	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(80.0f, 160.0f);	
	g_Body[0] = g_World.CreateBody(&bodyDef);
	g_Body[0]->CreateFixture(&fixtureDef);
	
	fixtureDef.shape = &box2;
	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(240.0f, 160.0f);
	g_Body[1] = g_World.CreateBody(&bodyDef);
	g_Body[1]->CreateFixture(&fixtureDef);
	
	float ratio = 1;
	b2PulleyJointDef jointDef;
	jointDef.Initialize(g_Body[0], g_Body[1], b2Vec2(80.0f, 10.0f), b2Vec2(240.0f, 10.0f),
						b2Vec2(80.0f, 160.0f), b2Vec2(240.0f, 160.0f), ratio);
	
	jointDef.maxLengthA = 200.0f;
	jointDef.maxLengthB = 200.0f;
		
	g_Joint = g_World.CreateJoint(&jointDef);
}

void RenderPulleyJoint()
{
	b2Vec2 position = g_Body[0]->GetPosition();
	float32 angle = g_Body[0]->GetAngle();
	DrawBox(position.x, position.y, angle*57.29577951f, 20, 20);
	
	position = g_Body[1]->GetPosition();
	angle = g_Body[1]->GetAngle();
	DrawBox(position.x, position.y, angle*57.29577951f, 30, 30);
}


@interface Box2DViewController ()
@property (nonatomic, retain) EAGLContext *context;@end

@implementation Box2DViewController

@synthesize animating, context;

- (void)awakeFromNib
{
    EAGLContext *aContext = nil;//[[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
    
    if (!aContext)
    {
        aContext = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
    }
    
    if (!aContext)
        NSLog(@"Failed to create ES context");
    else if (![EAGLContext setCurrentContext:aContext])
        NSLog(@"Failed to set ES context current");
    
	self.context = aContext;
	[aContext release];
	
    [(EAGLView *)self.view setContext:context];
    [(EAGLView *)self.view setFramebuffer];
    
    animating = FALSE;
    displayLinkSupported = FALSE;
    animationFrameInterval = 1;
    displayLink = nil;
    animationTimer = nil;
    
    // Use of CADisplayLink requires iOS version 3.1 or greater.
	// The NSTimer object is used as fallback when it isn't available.
    NSString *reqSysVer = @"3.1";
    NSString *currSysVer = [[UIDevice currentDevice] systemVersion];
    if ([currSysVer compare:reqSysVer options:NSNumericSearch] != NSOrderedAscending)
        displayLinkSupported = TRUE;
	
	b2BodyDef groundBodyDef;
	groundBodyDef.position.Set(160.0f, 400.0f);
	g_GroundBody = g_World.CreateBody(&groundBodyDef);
	b2PolygonShape groundBox;
	groundBox.SetAsBox(160.0f, 10.0f);
	b2FixtureDef fixtureDef;
	fixtureDef.shape = &groundBox;
	g_GroundBody->CreateFixture(&fixtureDef);
	
	//CreateDistanceJoint();
	//CreateRevoluteJoint();
	//CreatePrismaticJoint();
	CreatePulleyJoint();

}

- (void)dealloc
{
    if (program)
    {
        glDeleteProgram(program);
        program = 0;
    }
    
    // Tear down context.
    if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
    
    [context release];
    
    [super dealloc];
}

- (void)viewWillAppear:(BOOL)animated
{
    [self startAnimation];
    
    [super viewWillAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [self stopAnimation];
    
    [super viewWillDisappear:animated];
}

- (void)viewDidUnload
{
	[super viewDidUnload];
	
    if (program)
    {
        glDeleteProgram(program);
        program = 0;
    }

    // Tear down context.
    if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
	self.context = nil;	
}

- (NSInteger)animationFrameInterval
{
    return animationFrameInterval;
}

- (void)setAnimationFrameInterval:(NSInteger)frameInterval
{
    /*
	 Frame interval defines how many display frames must pass between each time the display link fires.
	 The display link will only fire 30 times a second when the frame internal is two on a display that refreshes 60 times a second. The default frame interval setting of one will fire 60 times a second when the display refreshes at 60 times a second. A frame interval setting of less than one results in undefined behavior.
	 */
    if (frameInterval >= 1)
    {
        animationFrameInterval = frameInterval;
        
        if (animating)
        {
            [self stopAnimation];
            [self startAnimation];
        }
    }
}

- (void)startAnimation
{
    if (!animating)
    {
        if (displayLinkSupported)
        {
            /*
			 CADisplayLink is API new in iOS 3.1. Compiling against earlier versions will result in a warning, but can be dismissed if the system version runtime check for CADisplayLink exists in -awakeFromNib. The runtime check ensures this code will not be called in system versions earlier than 3.1.
            */
            displayLink = [NSClassFromString(@"CADisplayLink") displayLinkWithTarget:self selector:@selector(drawFrame)];
            [displayLink setFrameInterval:animationFrameInterval];
            
            // The run loop will retain the display link on add.
            [displayLink addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
        }
        else
            animationTimer = [NSTimer scheduledTimerWithTimeInterval:(NSTimeInterval)((1.0 / 60.0) * animationFrameInterval) target:self selector:@selector(drawFrame) userInfo:nil repeats:TRUE];
        
        animating = TRUE;
    }
}

- (void)stopAnimation
{
    if (animating)
    {
        if (displayLinkSupported)
        {
            [displayLink invalidate];
            displayLink = nil;
        }
        else
        {
            [animationTimer invalidate];
            animationTimer = nil;
        }
        
        animating = FALSE;
    }
}

- (void)drawFrame
{
    [(EAGLView *)self.view setFramebuffer];
    
    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    
	g_World.Step(1.0f/60.0f, 10, 10);
    g_World.ClearForces();
	
	//RenderDistanceJoint();
	//RenderRevoluteJoint();
	//RenderPrismaticJoint();
	RenderPulleyJoint();
    
	b2Vec2 position = g_GroundBody->GetPosition();
    float32 angle = g_GroundBody->GetAngle();
	
	DrawBox(position.x, position.y, angle*57.29577951f, 160, 10);
       
    [(EAGLView *)self.view presentFramebuffer];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc. that aren't in use.
}

@end
