#import "game_block.h"
#import "me_Box2D.h"
#import "me_Common.h"

float g_ClearLine;

int CheckClearLine()
{
	int i,j;
	float y;
	float tx, ty;
	for(i=0;i<MAX_BODY;i++)
	{
		me_SBody *body = g_Box2D.GetBody(i);
		if(body->Type != ME_BOX2D_TYPE_BOX) continue;
		
		b2Vec2 position = body->body->GetPosition();
		float32 angle = body->body->GetAngle();	
		
		for(j=0;j<4;j++)
		{
			switch(j)
			{
				case 0: tx = -body->hx; ty = -body->hy; break;
				case 1: tx =  body->hx; ty = -body->hy; break;
				case 2: tx = -body->hx; ty =  body->hy; break;
				case 3: tx =  body->hx; ty =  body->hy; break;
			}
			
			y = ty * cos(angle) + tx * sin(angle);
			
			if((y + position.y)*16.0f < g_ClearLine) return 0;
		}
	}
	
	return 1;
}

void RenderClearLine()
{
	GLfloat vertex [4] =
	{
		0, g_ClearLine,
		480, g_ClearLine
	};
	
	glColor4f(1, 0, 0, 1);
	
	glDisable(GL_TEXTURE_2D);
	glVertexPointer(2, GL_FLOAT, 0, vertex);
	glEnableClientState(GL_VERTEX_ARRAY);	
	glDrawArrays(GL_LINES, 0, 2);	
	glEnable(GL_TEXTURE_2D);
	
	glColor4f(1, 1, 1, 1);
}

void LoadBlockScript(const char *FileName)
{
	const char *path = me_GetBundlePath(FileName, 0);
	FILE *fp;
	fp = fopen(path, "rt");
	if(fp==NULL)
	{
		NSLog(@"LoadBlockScript fail.");
		return;
	}
	
	char Text[256];
	float x, y, hx, hy, density, radius;
	while(!feof(fp))
	{
		fscanf(fp, "%s", Text);
		
		if(!strcmp(Text, "box"))
		{
			fscanf(fp, "%f %f %f %f %f", &x, &y, &hx, &hy, &density);
			fscanf(fp, "%s", Text);
			
			g_Box2D.CreateBox(x, y, hx, hy, Text, density);
			
			continue;
		}
		
		if(!strcmp(Text, "circle"))
		{
			fscanf(fp, "%f %f %f %f", &x, &y, &radius, &density);
			fscanf(fp, "%s", Text);
			
			g_Box2D.CreateCircle(x, y, radius, Text, density);
			
			continue;
		}
		
		if(!strcmp(Text, "clearline"))
		{
			fscanf(fp, "%f", &g_ClearLine);
			
			continue;
		}
	}
	
	fclose(fp);	
}