#import "me_box2d.h"
#import "me_math.h"
#import <OpenGLES/EAGL.h>

#import <OpenGLES/ES1/gl.h>
#import <OpenGLES/ES1/glext.h>
#import <OpenGLES/ES2/gl.h>
#import <OpenGLES/ES2/glext.h>

me_CBox2D g_Box2D;

me_CBox2D::me_CBox2D()
{
	m_World = NULL;
}

me_CBox2D::~me_CBox2D()
{
	Clear();
}

void me_CBox2D::Clear()
{
	delete m_World;
	m_World=NULL;
}

void me_CBox2D::Init()
{
	b2Vec2 gravity(0.0f, 98.0f/16.0f);	
	bool doSleep = true;
	m_World = new b2World(gravity, doSleep);
	
	b2BodyDef groundBodyDef;
	groundBodyDef.position.Set(240.0f/16.0f, 320.0f/16.0f);
	b2Body *g_GroundBody = m_World->CreateBody(&groundBodyDef);
	b2PolygonShape groundBox;
	groundBox.SetAsBox(500.0f/16.0f, 10.0f/16.0f);
	g_GroundBody->CreateFixture(&groundBox, 0);
}

void me_CBox2D::CreateBox(float x, float y, float hx, float hy, const char *texname, float density)
{
	int i;
	for(i=0;i<MAX_BODY;i++)
	{
		if(m_Body[i].Type != ME_BOX2D_TYPE_NONE) continue;
		
		m_Body[i].Type = ME_BOX2D_TYPE_BOX;
		m_Body[i].hx = hx/16.0f;
		m_Body[i].hy = hy/16.0f;
		
		b2BodyDef bodyDef;
		bodyDef.type = b2_dynamicBody;
		bodyDef.position.Set(x/16.0f, y/16.0f);	
		bodyDef.angularDamping = 0.02f;
		m_Body[i].body = m_World->CreateBody(&bodyDef);
		
		b2PolygonShape dynamicBox;	
		dynamicBox.SetAsBox(hx/16.0f, hy/16.0f);
		b2MassData massData;
		dynamicBox.ComputeMass(&massData, density);
		
		b2FixtureDef fixtureDef;
		fixtureDef.shape = &dynamicBox;
		fixtureDef.density = density;
		fixtureDef.friction = 1.0f;
		
		m_Body[i].body->CreateFixture(&fixtureDef);	
		m_Body[i].body->SetMassData(&massData);
		
		m_Body[i].tex = me_CreateTexture(texname);
		break;
	}
}


void me_CBox2D::CreateCircle(float x, float y, float radius, const char *texname, float density)
{
	int i;
	for(i=0;i<MAX_BODY;i++)
	{
		if(m_Body[i].Type != ME_BOX2D_TYPE_NONE) continue;
		
		m_Body[i].Type = ME_BOX2D_TYPE_CIRCLE;
		m_Body[i].radius = radius/16.0f;
		
		b2BodyDef bodyDef;
		bodyDef.type = b2_dynamicBody;
		bodyDef.position.Set(x/16.0f, y/16.0f);	
		bodyDef.angularDamping = 0.02f;
		m_Body[i].body = m_World->CreateBody(&bodyDef);
		
		b2CircleShape dynamicCircle;	
		dynamicCircle.m_p.Set(0, 0);
		dynamicCircle.m_radius = radius/16.0f;
		b2MassData massData;
		dynamicCircle.ComputeMass(&massData, density);
		
		b2FixtureDef fixtureDef;
		fixtureDef.shape = &dynamicCircle;
		fixtureDef.density = density;
		fixtureDef.friction = 1.0f;
		
		m_Body[i].body->CreateFixture(&fixtureDef);	
		m_Body[i].body->SetMassData(&massData);
		
		m_Body[i].tex = me_CreateTexture(texname);
		break;
	}
}

me_SBody *me_CBox2D::GetBody(int Index)
{
	return &m_Body[Index];
}

void me_CBox2D::SetForce(int Index, float fx, float fy)
{
	b2Vec2 f = b2Vec2( fx / 5.0f , fy / 5.0f);
	
	m_Body[Index].body->SetAwake(true);
	m_Body[Index].body->SetLinearVelocity(f);
}

void me_CBox2D::FrameMove()
{
	m_World->Step(1.0f/60.0f, 6, 2);
    m_World->ClearForces();
}

void me_CBox2D::DrawBox(float x, float y, float angle, float hx, float hy, me_STexture tex)
{
	GLfloat vertex[8] =
	{
		-hx,	-hy,
		hx,		-hy,
		-hx,	hy,
		hx,		hy
	};
	
	GLfloat texcoord[8] = {
		0, 0,
		1, 0,
		0, 1,
		1, 1
	}; 
	
	glPushMatrix();
	glTranslatef(x, y, 0);
	glRotatef(angle, 0, 0, 1);
	
	glBindTexture(GL_TEXTURE_2D, tex.TexID);
	
	glVertexPointer(2, GL_FLOAT, 0, vertex);
	glEnableClientState(GL_VERTEX_ARRAY);
	
	glTexCoordPointer(2, GL_FLOAT, 0, texcoord);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);	
	
	glPopMatrix();
}

void me_CBox2D::Render()
{
	int i;
	for(i=0;i<MAX_BODY;i++)
	{
		if(m_Body[i].Type == ME_BOX2D_TYPE_NONE) continue;
		
		b2Vec2 position = m_Body[i].body->GetPosition();
		float32 angle = m_Body[i].body->GetAngle();	
		
		switch(m_Body[i].Type)
		{
			case ME_BOX2D_TYPE_BOX:
				DrawBox(position.x*16.0f, position.y*16.0f, RAD2DEGREE(angle), m_Body[i].hx*16.0f, m_Body[i].hy*16.0f, m_Body[i].tex);
				break;
			case ME_BOX2D_TYPE_CIRCLE:
				DrawBox(position.x*16.0f, position.y*16.0f, RAD2DEGREE(angle), m_Body[i].radius*16.0f, m_Body[i].radius*16.0f, m_Body[i].tex);
				break;
		}		
	}
}