#import "me_Particle.h"
#import "me_Timer.h"
#import "me_Common.h"

me_CParticleDataMgr g_ParticleDataMgr;

void me_CParticleDataMgr::Add(me_SParticle Data)
{
	int i;
	for(i=0;i<MAX_PARTICLE;i++)
	{
		if(ParticleData[i].Life <= 0.0f)
		{
			ParticleData[i] = Data;
			return;
		}
	}
}

void me_CParticleDataMgr::Render()
{
	int i;
	for(i=0;i<MAX_PARTICLE;i++)
	{
		if(ParticleData[i].Life<= 0.0f) continue;
		
		ParticleData[i].Life -= me_GetElapsedTime();
		vec2add(ParticleData[i].Pos, ParticleData[i].Pos, ParticleData[i].PosDelta);
		
		ParticleData[i].Angle += ParticleData[i].AngleDelta;
		ParticleData[i].Scale += ParticleData[i].ScaleDelta;
		
		vec4add(ParticleData[i].Color, ParticleData[i].Color, ParticleData[i].ColorDelta);
		if(ParticleData[i].Color[3]<0.0f) ParticleData[i].Life = 0;
		
		glColor4f(ParticleData[i].Color[0], ParticleData[i].Color[1], ParticleData[i].Color[2], ParticleData[i].Color[3]);
		me_PutSpr(ParticleData[i].Pos[0]-((float)ParticleData[i].Tex.Width/2.0f*ParticleData[i].Scale), 
				  ParticleData[i].Pos[1]-((float)ParticleData[i].Tex.Height/2.0f*ParticleData[i].Scale),
				  ParticleData[i].Tex, ParticleData[i].Angle, ParticleData[i].Scale);
	}
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
me_CParticle::me_CParticle()
{
	vec2zero(m_PosMin);
	vec2zero(m_PosMax);
	vec2zero(m_PosDeltaMin);
	vec2zero(m_PosDeltaMax);
	vec2zero(m_PosAccel);
	
	m_AngleMax=0; 
	m_AngleMin=0;	
	m_AngleDelta=0;
	
	m_ScaleMax=0;
	m_ScaleMin=0;
	m_ScaleDelta=0;
	
	vec4one(m_Color);
	vec4one(m_ColorDelta);
	
	m_LifeMax=m_LifeMin=0;
	
	m_CreateCount=0;
	m_Delay=0;	
	m_CurrentDelay=0;
}

me_CParticle::~me_CParticle()
{
	Release();
}

void me_CParticle::Release()
{
	me_DeleteTexture(m_Tex);
}

void me_CParticle::Load(const char *FileName)
{
	const char *path = me_GetBundlePath(FileName, 0);
	FILE *fp;
	fp = fopen(path, "rt");
	if(fp==NULL)
	{
		NSLog(@"me_CParticle::Load load fail.");
		return;
	}
	
	char Text[256];
	while(!feof(fp))
	{
		fscanf(fp, "%s", Text);
		
		if(!strcmp(Text, "texname"))
		{
			fscanf(fp, "%s", Text);
			me_DeleteTexture(m_Tex);
			m_Tex = me_CreateTexture(Text);
			continue;
		}
		
		if(!strcmp(Text, "pos"))
		{
			fscanf(fp, "%f %f %f %f", &m_PosMin[0], &m_PosMin[1], &m_PosMax[0], &m_PosMax[1]);
			continue;
		}
		
		if(!strcmp(Text, "posdelta"))
		{
			fscanf(fp, "%f %f %f %f", &m_PosDeltaMin[0], &m_PosDeltaMin[1], &m_PosDeltaMax[0], &m_PosDeltaMax[1]);
			continue;
		}
		
		if(!strcmp(Text, "posaccel"))
		{
			fscanf(fp, "%f %f", &m_PosAccel[0], &m_PosAccel[1]);
			continue;
		}
		
		if(!strcmp(Text, "angle"))
		{
			char str[256];
			fscanf(fp, "%s", str);
			
			if(!strcmp(str, "direction"))
			{
				m_AngleMin = m_AngleMax = -1;
			}
			else
			{
				m_AngleMin = atof(str);
				fscanf(fp, "%f", &m_AngleMax);
			}
			continue;
		}
		
		if(!strcmp(Text, "angledelta"))
		{
			fscanf(fp, "%f", &m_AngleDelta);
			continue;
		}
		
		if(!strcmp(Text, "scale"))
		{
			fscanf(fp, "%f %f", &m_ScaleMin, &m_ScaleMax);
			continue;
		}
		
		if(!strcmp(Text, "scaledelta"))
		{
			fscanf(fp, "%f", &m_ScaleDelta);
			continue;
		}
		
		if(!strcmp(Text, "createcount"))
		{
			fscanf(fp, "%d", &m_CreateCount);
			continue;
		}
		
		if(!strcmp(Text, "delay"))
		{
			fscanf(fp, "%f", &m_Delay);
			continue;
		}
		
		if(!strcmp(Text, "life"))
		{
			fscanf(fp, "%f %f", &m_LifeMin, &m_LifeMax);
			continue;
		}
		
		if(!strcmp(Text, "color"))
		{
			fscanf(fp, "%f %f %f %f", &m_Color[0], &m_Color[1], &m_Color[2], &m_Color[3]);
			continue;
		}
		
		if(!strcmp(Text, "colordelta"))
		{
			fscanf(fp, "%f %f %f %f", &m_ColorDelta[0], &m_ColorDelta[1], &m_ColorDelta[2], &m_ColorDelta[3]);
			continue;
		}
	}
	
	fclose(fp);	
}

void me_CParticle::Process(vector2f Pos)
{
	if(m_CreateCount==0) return;
	
	m_CurrentDelay-= me_GetElapsedTime();
	if(m_CurrentDelay>0) return;

	m_CurrentDelay = m_Delay;
	
	int i;
	me_SParticle tData;
	for(i=0;i<m_CreateCount;i++)
	{
		tData.Pos[0] = m_PosMin[0] + (m_PosMax[0] - m_PosMin[0]) * me_GetRand();
		tData.Pos[1] = m_PosMin[1] + (m_PosMax[1] - m_PosMin[1]) * me_GetRand();
		
		vec2add(tData.Pos, tData.Pos, Pos);
		
		tData.PosDelta[0] = m_PosDeltaMin[0] + (m_PosDeltaMax[0] - m_PosDeltaMin[0]) * me_GetRand();
		tData.PosDelta[1] = m_PosDeltaMin[1] + (m_PosDeltaMax[1] - m_PosDeltaMin[1]) * me_GetRand();
		vec2copy(tData.PosAccel, m_PosAccel);
		
		tData.Angle = m_AngleMin + (m_AngleMax - m_AngleMin) * me_GetRand();
		if(tData.Angle == -1)
		{
			vector2f oPos = { 0, 0 };
			tData.Angle = (GetAngle(oPos, tData.PosDelta)) ;
		}
		tData.AngleDelta = m_AngleDelta;
		
		tData.Scale = m_ScaleMin + (m_ScaleMax - m_ScaleMin) * me_GetRand();
		tData.ScaleDelta = m_ScaleDelta;
		
		vec4copy(tData.Color, m_Color);
		vec4copy(tData.ColorDelta, m_ColorDelta);
		
		tData.Life = m_LifeMin + (m_LifeMax - m_LifeMin) * me_GetRand();
		tData.Tex = m_Tex;
		
		g_ParticleDataMgr.Add(tData);
	}
}