#import "me_gui.h"
#import "me_Sprite.h"
#import "me_common.h"

me_CGUI_Image::me_CGUI_Image(float x, float y, char *TexName)
{
	Pos[0] = x;
	Pos[1] = y;
	Tex = me_CreateTexture(TexName);
	
	Type = ME_GT_IMAGE;
}

void me_CGUI_Image::Render()
{
	me_PutSpr(Pos[0], Pos[1], Tex);
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

me_CGUI_Button::me_CGUI_Button(float x, float y, char *TexName, char *cmd)
{
	Pos[0] = x;
	Pos[1] = y;
	Tex = me_CreateTexture(TexName);
	strcpy(Command, cmd);
	
	Type = ME_GT_BUTTON;
}

void me_CGUI_Button::Render()
{
	me_PutSpr(Pos[0], Pos[1], Tex);
}

char *me_CGUI_Button::Check(float x, float y)
{
	if(x > Pos[0] && x < Pos[0] + Tex.Width &&
	   y > Pos[1] && y < Pos[1] + Tex.Height) return Command;
	
	return NULL;
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

me_CGUI_Listbox::me_CGUI_Listbox(float x, float y, float width, float height, char *cmd)
{
	Pos[0] = x;
	Pos[1] = y;
	
	Width = width;
	Height = height;
	
	strcpy(Command, cmd);
	
	Type = ME_GT_LISTBOX;
	
	ItemCount=0;
}

void me_CGUI_Listbox::AddItem(char *item)
{
	Item[ItemCount]=me_CreateTexture(item);
	ItemCount ++;
}

void me_CGUI_Listbox::ClearItem()
{
	int i;
	for(i=0;i<ItemCount;i++)
	{
		me_DeleteTexture(Item[i]);
	}
	
	ItemCount = 0;
}


void me_CGUI_Listbox::Render()
{
	int i;
	float x = Pos[0], y = Pos[1];
	for(i=0;i<ItemCount;i++)
	{
		me_PutSpr(x, y, Item[i]);
		
		x += Item[i].Width + 10;
		if( x >= Pos[0] + Width)
		{
			x = Pos[0];
			y+= Item[i].Height + 10;
		}
	}	
}

char *me_CGUI_Listbox::Check(float x, float y)
{	
	int i;
	float sx = Pos[0], sy = Pos[1];
	for(i=0;i<ItemCount;i++)
	{
		if(x > sx && x < sx + Item[i].Width &&
		   y > sy && y < sy + Item[i].Height) 
		{
			sprintf(ReturnText, "%s %d", Command, i);
			return ReturnText;
		}
		
		sx += Item[i].Width + 10;
		if( sx >= Pos[0] + Width)
		{
			sx = Pos[0];
			sy += Item[i].Height + 10;
		}
	}	
	
	return NULL;
}

///////////////////////////////////////////////////////////////
me_CGUI_Scene::me_CGUI_Scene()
{
	int i;
	for(i=0;i<MAX_SCENECONTROL;i++)
	{
		m_Control[i]=NULL;
	}
}

me_CGUI_Scene::~me_CGUI_Scene()
{
	Clear();
}

void me_CGUI_Scene::Clear()
{
	int i;
	for(i=0;i<MAX_SCENECONTROL;i++)
	{
		if(m_Control[i])
		{
			me_DeleteTexture(m_Control[i]->Tex);
			
			if(m_Control[i]->Type == ME_GT_LISTBOX) 
				((me_CGUI_Listbox*)m_Control[i])->ClearItem();
			
			delete [] m_Control[i];
			m_Control[i] = NULL;
		}
	}
}

void me_CGUI_Scene::AddControl(me_CGUI_Control *control)
{
	int i;
	for(i=0;i<MAX_SCENECONTROL;i++)
	{
		if(m_Control[i]==NULL)
		{
			m_Control[i]=control;
			return;
		}
	}
}

void me_CGUI_Scene::Load(const char *filename)
{
	const char *path = me_GetBundlePath(filename, 0);
	FILE *fp;
	fp = fopen(path, "rt");
	if(fp==NULL)
	{
		NSLog(@"me_CGUI_Scene::Load fail.");
		return ;
	}
	
	float x, y, width, height;
	char Text[256], Text2[256];
	while(!feof(fp))
	{
		fscanf(fp, "%s", Text);
		
		if(!strcmp(Text, "image"))
		{			
			fscanf(fp, "%f %f %s"	, &x, &y, Text);
			
			me_CGUI_Image *tImage=new me_CGUI_Image(x, y, Text);
			AddControl(tImage);
			
			continue;
		}
		
		if(!strcmp(Text, "button"))
		{
			fscanf(fp, "%f %f %s %s"	, &x, &y, Text, Text2);
			
			me_CGUI_Button *tButton=new me_CGUI_Button(x, y, Text, Text2);
			AddControl(tButton);
			
			continue;
		}
		
		if(!strcmp(Text, "listbox"))
		{
			fscanf(fp, "%f %f %f %f %s"	, &x, &y, &width, &height, Text);
			
			me_CGUI_Listbox *tListbox=new me_CGUI_Listbox(x, y, width, height, Text);
			
			while(1)
			{
				fscanf(fp, "%s", Text);
				if(!strcmp(Text, "end")) break;
				
				tListbox->AddItem(Text);
			}
			
			AddControl(tListbox);
			
			continue;
		}
	}
	
	fclose(fp);	
	
	return;
}


void me_CGUI_Scene::Render()
{
	int i;
	for(i=0;i<MAX_SCENECONTROL;i++)
	{
		if(m_Control[i])
			m_Control[i]->Render();
	}
	
	return ;
}

int me_CGUI_Scene::CommandAnalyze(char *Text)
{
	if(Text[0]=='@')
	{
		Clear();
		Load(Text+1);
		
		return 1;
	}
	
	return 0;
}

char *me_CGUI_Scene::TouchEnd(float x, float y)
{
	m_LastCommand[0]=NULL;
	
	int i;
	for(i=0;i<MAX_SCENECONTROL;i++)
	{
		if(m_Control[i]==NULL) continue;
		
		switch(m_Control[i]->Type)
		{
			case ME_GT_BUTTON:
			{
				char *cmd = ((me_CGUI_Button*)m_Control[i])->Check(x, y);
				if(cmd!=NULL)
					strcpy(m_LastCommand, cmd);
					
				break;
			}
				
			case ME_GT_LISTBOX:
			{
				char *cmd = ((me_CGUI_Listbox*)m_Control[i])->Check(x, y);
				if(cmd!=NULL)
					strcpy(m_LastCommand, cmd);
				
				break;
			}
		}
	}
	
	if(CommandAnalyze(m_LastCommand))
		m_LastCommand[0]=NULL;
	
	return m_LastCommand;
}