//
//  NeonFightersViewController.m
//  NeonFighters
//
//  Created by 영호 천 on 10. 10. 22..
//  Copyright 2010 .. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>

#import "NeonFightersViewController.h"
#import "EAGLView.h"

#import "me_Timer.h"
#import "me_Texture.h"
#import "me_Sprite.h"
#import "me_ScreenPad.h"
#import "game_common.h"
#import "game_player.h"
#import "game_bg.h"
#import "game_Enemy.h"
#import "game_Bullet.h"
#import "game_effect.h"
#import "game_saveload.h"
#import "game_Respawn.h"

@interface NeonFightersViewController ()
@property (nonatomic, retain) EAGLContext *context;
@end

@implementation NeonFightersViewController

@synthesize animating, context;

- (void)awakeFromNib
{
    EAGLContext *aContext = nil;[[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
    
    if (!aContext)
    {
        aContext = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
    }
    
    if (!aContext)
        NSLog(@"Failed to create ES context");
    else if (![EAGLContext setCurrentContext:aContext])
        NSLog(@"Failed to set ES context current");
    
	self.context = aContext;
	[aContext release];
	
    [(EAGLView *)self.view setContext:context];
    [(EAGLView *)self.view setFramebuffer];
  
    animating = FALSE;
    displayLinkSupported = FALSE;
    animationFrameInterval = 1;
    displayLink = nil;
    animationTimer = nil;
    
    // Use of CADisplayLink requires iOS version 3.1 or greater.
	// The NSTimer object is used as fallback when it isn't available.
    NSString *reqSysVer = @"3.1";
    NSString *currSysVer = [[UIDevice currentDevice] systemVersion];
    if ([currSysVer compare:reqSysVer options:NSNumericSearch] != NSOrderedAscending)
        displayLinkSupported = TRUE;
	
	g_NeonLineTex = me_CreateTexture(@"neonline.png");
	glEnable(GL_TEXTURE_2D);
	
	me_Update();
		
	g_Player.Init();
	g_BG.Init();
	g_Enemy.Init();
	
	//g_Enemy.Create(0, 0, ET_CORE);
	//g_Enemy.Create(0, 0, ET_ARROW);
	//g_Enemy.Create(0, 0, ET_OCTAGON);
	
	g_Bullet.Init();
	
	g_Exp.Load("Explosion.par");
	
	g_RespawnPattern.Load("respawn.ptn");
	
	Load();
}

- (void)dealloc
{
    if (program)
    {
        glDeleteProgram(program);
        program = 0;
    }
    
    // Tear down context.
    if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
    
    [context release];
    
    [super dealloc];
}

- (void)viewWillAppear:(BOOL)animated
{
    [self startAnimation];
    
    [super viewWillAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [self stopAnimation];
    
    [super viewWillDisappear:animated];
}

- (void)viewDidUnload
{
	[super viewDidUnload];
	
    if (program)
    {
        glDeleteProgram(program);
        program = 0;
    }

    // Tear down context.
    if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
	self.context = nil;	
}

- (NSInteger)animationFrameInterval
{
    return animationFrameInterval;
}

- (void)setAnimationFrameInterval:(NSInteger)frameInterval
{
    /*
	 Frame interval defines how many display frames must pass between each time the display link fires.
	 The display link will only fire 30 times a second when the frame internal is two on a display that refreshes 60 times a second. The default frame interval setting of one will fire 60 times a second when the display refreshes at 60 times a second. A frame interval setting of less than one results in undefined behavior.
	 */
    if (frameInterval >= 1)
    {
        animationFrameInterval = frameInterval;
        
        if (animating)
        {
            [self stopAnimation];
            [self startAnimation];
        }
    }
}

- (void)startAnimation
{
    if (!animating)
    {
        if (displayLinkSupported)
        {
            /*
			 CADisplayLink is API new in iOS 3.1. Compiling against earlier versions will result in a warning, but can be dismissed if the system version runtime check for CADisplayLink exists in -awakeFromNib. The runtime check ensures this code will not be called in system versions earlier than 3.1.
            */
            displayLink = [NSClassFromString(@"CADisplayLink") displayLinkWithTarget:self selector:@selector(drawFrame)];
            [displayLink setFrameInterval:animationFrameInterval];
            
            // The run loop will retain the display link on add.
            [displayLink addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
        }
        else
            animationTimer = [NSTimer scheduledTimerWithTimeInterval:(NSTimeInterval)((1.0 / 60.0) * animationFrameInterval) target:self selector:@selector(drawFrame) userInfo:nil repeats:TRUE];
        
        animating = TRUE;
    }
}

- (void)stopAnimation
{
    if (animating)
    {
        if (displayLinkSupported)
        {
            [displayLink invalidate];
            displayLink = nil;
        }
        else
        {
            [animationTimer invalidate];
            animationTimer = nil;
        }
        
        animating = FALSE;
    }
}

- (void)drawFrame
{
    [(EAGLView *)self.view setFramebuffer];
    
    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
	
	me_Update();
	g_Player.FrameMove();
	g_Enemy.FrameMove();
	g_Bullet.FrameMove();
	g_RespawnPattern.Process();
	
	g_BG.RenderBefore();
	
	vector2f P_Pos;
	g_Player.GetPos(P_Pos);
	
	g_BG.Collide(P_Pos);
	g_Player.SetPos(P_Pos);
	
	glPushMatrix();
	glTranslatef(240, 160, 0);
	glScalef(0.5f, 0.5f, 0.5f);
	glTranslatef(-P_Pos[0], -P_Pos[1], 0);
    
	g_BG.RenderAfter();
	g_Player.Render();
	g_Enemy.Render();
	g_Bullet.Render();
	g_ParticleDataMgr.Render();
	
	glPopMatrix();
	g_ScreenPad.Render();
	
    [(EAGLView *)self.view presentFramebuffer];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc. that aren't in use.
}

-(BOOL) shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation
{
	if(toInterfaceOrientation == UIInterfaceOrientationLandscapeLeft ||
	   toInterfaceOrientation == UIInterfaceOrientationLandscapeRight)
	{
		return YES;
	}
	
	return NO;
}

@end
