#import "game_Bullet.h"
#import "game_Enemy.h"
#import "game_Player.h"
#import "game_common.h"
#import "game_effect.h"
#import "me_Timer.h"

CBullet g_Bullet;

void CBullet::Init()
{
	m_Neon[BT_NORMAL].Load("bt_normal.neon");
	m_Neon[BT_ENEMY].Load("bt_enemy.neon");
	
	vector4f Color = { 1 , 0, 0, 1 };
	m_Neon[BT_ENEMY].SetColor(Color);
}

void CBullet::Create(vector2f Pos, float Angle, float Speed, int Type)
{
	int i;
	int ms=0, me=40;
	
	if(Type == BT_ENEMY)
	{
		ms = 40;
		me = 128;
	}
	
	for(i=ms;i<me;i++)
	{
		if(m_Bullet[i].Life<=0.0f)
		{
			vec2copy(m_Bullet[i].Pos, Pos);
			m_Bullet[i].Angle = Angle;
			
			float rad = DEGREE2RAD(Angle);
			float tx = 0, ty = -Speed;	
			m_Bullet[i].Delta[0] = tx * cos(rad) - ty * sin(rad);
			m_Bullet[i].Delta[1] = ty * cos(rad) + tx * sin(rad);
			
			vec2add(m_Bullet[i].Pos, m_Bullet[i].Pos, m_Bullet[i].Delta);
			
			m_Bullet[i].Type = Type;
			if(m_Bullet[i].Type == BT_ENEMY) m_Bullet[i].Life = 8.0f;
			else							 m_Bullet[i].Life = 0.8f;
			
			break;
		}
	}
}

void CBullet::FrameMove()
{
	int i;
	for(i=0;i<MAX_BULLET;i++)
	{
		if(m_Bullet[i].Life<=0.0f) continue;

		vec2add(m_Bullet[i].Pos, m_Bullet[i].Pos, m_Bullet[i].Delta);
		m_Bullet[i].Life -= me_GetElapsedTime();
		
		if(i<40)
		{
			if(g_Enemy.CheckBullet(m_Bullet[i].Pos))
			{
				g_Exp.Process(m_Bullet[i].Pos);
				m_Bullet[i].Life = 0;
				continue;
			}
		}
		else
		{
			if(g_Player.CheckBullet(m_Bullet[i].Pos))
			{
				g_Exp.Process(m_Bullet[i].Pos);
				m_Bullet[i].Life = 0;
				continue;
			}
		}
	}
}

void CBullet::Render()
{
	int i;
	for(i=0;i<MAX_BULLET;i++)
	{
		if(m_Bullet[i].Life<=0.0f) continue;
		
		glPushMatrix();
		
		glTranslatef(m_Bullet[i].Pos[0], m_Bullet[i].Pos[1], 0);
		glRotatef(m_Bullet[i].Angle,0,0, 1);
		
		m_Neon[m_Bullet[i].Type].Render(g_NeonLineTex.TexID);
		
		glPopMatrix();
	}
}

void CBullet::Save(FILE *fp)
{
	fwrite(m_Bullet, sizeof(SBullet), MAX_BULLET, fp);
}

void CBullet::Load(FILE *fp)
{
	fread(m_Bullet, sizeof(SBullet), MAX_BULLET, fp);
}