#import "game_enemy.h"
#import "game_bg.h"
#import "game_common.h"
#import "game_Player.h"
#import "game_Pattern.h"
#import "game_Bullet.h"
#import "me_Common.h"
#import "me_timer.h"

CEnemy g_Enemy;

CEnemy::CEnemy()
{
}

CEnemy::~CEnemy()
{
}

void CEnemy::Init()
{
	m_Pattern[ET_ARROW].Load("arrow.ptn");
	m_Pattern[ET_OCTAGON].Load("octagon.ptn");
	m_Pattern[ET_CORE].Load("core.ptn");
	
	vector4f Color={1, 1, 1, 1};
	m_Neon[ET_ARROW].Load("arrow.neon");
	Color[0] = 0.8823f; Color[1] = 0.2274f; Color[2] = 0.3803f;
	m_Neon[ET_ARROW].SetColor(Color);
	
	m_Neon[ET_OCTAGON].Load("octagon.neon");
	Color[0] = 0.9019f; Color[1] = 0.5411f; Color[2] = 0.0235f;
	m_Neon[ET_OCTAGON].SetColor(Color);
	
	m_Neon[ET_CORE].Load("core.neon");
	Color[0] = 0.2705f; Color[1] = 0.3333f; Color[2] = 0.9294f;
	m_Neon[ET_CORE].SetColor(Color);
}

void CEnemy::Create(float x, float y, int Type)
{
	int i;
	
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type == ET_NONE)
		{
			m_Enemy[i].Pos[0] = x;
			m_Enemy[i].Pos[1] = y;
			m_Enemy[i].Angle = 0;
			m_Enemy[i].Type = Type;
			
			m_Enemy[i].MoveTime = 0;			
			m_Enemy[i].PatternIdx = 0;
			break;
		}
	}
}

int CEnemy::CheckBullet(vector2f bPos)
{
	int i;
	vector2f delta;
	float Length;
	
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type == ET_NONE) continue;
		
		vec2sub(delta, m_Enemy[i].Pos, bPos);
		Length = vec2length(delta);
		
		if(Length <= 17 )
		{
			m_Enemy[i].Type = ET_NONE;
			return 1;
		}
	}
	
	return 0;
}

void CEnemy::FrameMove()
{
	int i;
	
	vector2f playerpos;
	g_Player.GetPos(playerpos);
	
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type == ET_NONE) continue;
		
		if(m_Enemy[i].MoveTime>0)
		{
			m_Enemy[i].MoveTime -= me_GetElapsedTime();
			
			vec2add(m_Enemy[i].Pos, m_Enemy[i].Pos, m_Enemy[i].TargetDelta);
			
			g_BG.Collide(m_Enemy[i].Pos);
			continue;
		}
		
		while(1)
		{
			SPattern *pattern = m_Pattern[m_Enemy[i].Type].GetPattern(m_Enemy[i].PatternIdx);
		
			if(pattern->Command==EP_LOOKPLAYER)
			{
				float targetAngle = GetAngle(m_Enemy[i].Pos, playerpos);
					
				if(fabsf(targetAngle - m_Enemy[i].Angle) < pattern->Value[0])
				{
					m_Enemy[i].Angle = targetAngle;
				}
				else 
				{
					if( fabs(targetAngle - m_Enemy[i].Angle) < 180.0f)
					{
						if(targetAngle > m_Enemy[i].Angle) m_Enemy[i].Angle += pattern->Value[0];
						if(targetAngle < m_Enemy[i].Angle) m_Enemy[i].Angle -= pattern->Value[0];
					}
					else
					{
						if(targetAngle > m_Enemy[i].Angle) m_Enemy[i].Angle -= pattern->Value[0];
						if(targetAngle < m_Enemy[i].Angle) m_Enemy[i].Angle += pattern->Value[0];
					}
						
					if(m_Enemy[i].Angle < 0) m_Enemy[i].Angle += 360;
					if(m_Enemy[i].Angle > 360) m_Enemy[i].Angle -= 360;
				}
					
				float rad = DEGREE2RAD(m_Enemy[i].Angle);
				float tx = 0, ty = -5;
				m_Enemy[i].TargetDelta[0] = tx * cos(rad) - ty * sin(rad);
				m_Enemy[i].TargetDelta[1] = ty * cos(rad) + tx * sin(rad);
			}
				
			if(pattern->Command==EP_MOVEFORWARD)
			{
				m_Enemy[i].MoveTime = pattern->Value[0];
				break;
			}
			
			if(pattern->Command==EP_RANDOMMOVE)
			{
				m_Enemy[i].Angle = me_GetRand() * 360.0f;
				
				float rad = DEGREE2RAD(m_Enemy[i].Angle);
				float tx = 0, ty = -5;
				m_Enemy[i].TargetDelta[0] = tx * cos(rad) - ty * sin(rad);
				m_Enemy[i].TargetDelta[1] = ty * cos(rad) + tx * sin(rad);
				
				m_Enemy[i].MoveTime = pattern->Value[0];
				break;
			}
			
			if(pattern->Command==EP_FIRE)
			{
				if(m_Enemy[i].FireDelay > 0)
				{
					m_Enemy[i].FireDelay -= me_GetElapsedTime();
				}
				else 
				{
					float angle = pattern->Value[0];
					if(angle == -1)
					{
						angle = GetAngle(m_Enemy[i].Pos, playerpos);
					}
					
					g_Bullet.Create(m_Enemy[i].Pos, angle, 5, BT_ENEMY);
					
					m_Enemy[i].FireDelay = pattern->Value[1];
				}				
			}
		}
	}
}

void CEnemy::Render()
{
	int i;
	
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type == ET_NONE) continue;
		
		glPushMatrix();
		glTranslatef(m_Enemy[i].Pos[0], m_Enemy[i].Pos[1], 0);
		glRotatef(m_Enemy[i].Angle, 0, 0, 1);
		
		m_Neon[m_Enemy[i].Type].Render(g_NeonLineTex.TexID);
		
		glPopMatrix();
	}
}

void CEnemy::Save(FILE *fp)
{
	fwrite(m_Enemy, sizeof(SEnemy), MAX_ENEMY, fp);
}

void CEnemy::Load(FILE *fp)
{
	fread(m_Enemy, sizeof(SEnemy), MAX_ENEMY, fp);
}