#import "game_neon.h"

CNeon::CNeon()
{
	nVertexList = 0;
	
	m_Color[0]=1.0f;
	m_Color[1]=1.0f;
	m_Color[2]=1.0f;
	m_Color[3]=1.0f;
}

CNeon::~CNeon()
{
}

void CNeon::Render(GLuint TexID)
{
	glBindTexture(GL_TEXTURE_2D, TexID);
	
	glVertexPointer(2, GL_FLOAT, 0, rVL);
	glTexCoordPointer(2, GL_FLOAT, 0,rTL);
	
	glColor4f(m_Color[0], m_Color[1], m_Color[2], m_Color[3]);
    
    glDrawArrays(GL_TRIANGLES, 0, nVertexList*3);
	
	glColor4f(1, 1, 1, 1);
}

void CNeon::Collide(vector2f Pos, float Radius)
{
	int i;
	float a, b, c;
	vector2f N_Pos;
	
	for(i=0;i<nVertexList;i+=2)
	{
		a = NormalList[i/2][0];
		b = NormalList[i/2][1];
		c = NormalList[i/2][2];
		
		N_Pos[0] = Pos[0] - a * Radius;
		N_Pos[1] = Pos[1] - b * Radius;
		
		float Sign = a * N_Pos[0] + b * N_Pos[1] + c;
		
		if(Sign < 0)
		{
			Pos[0] -= a * Sign;
			Pos[1] -= b * Sign;
		}
	}
}

void CNeon::Load(const char *filename)
{
	const char *path = me_GetBundlePath(filename, 0);
	FILE *fp;
	fp = fopen(path, "rt");
	if(fp==NULL)
	{
		NSLog(@"cNEON::Load load fail.");
		return;
	}
	
	fscanf(fp, "%d", &nVertexList);
	
	int i;
	for(i=0;i<nVertexList;i++)
	{
		fscanf(fp, "%f %f", &VertexList[i][0], &VertexList[i][1]);
	}
	
	fclose(fp);
	
	for(i=0;i<nVertexList;i+=2)
	{
		NormalList[i/2][0] = VertexList[i][1] - VertexList[i+1][1];
		NormalList[i/2][1] = VertexList[i+1][0] - VertexList[i][0];
		
		vec2normalize(NormalList[i/2]);
		
		NormalList[i/2][2] = -(NormalList[i/2][0] * VertexList[i][0] + NormalList[i/2][1] * VertexList[i][1]);
	}
	
	MakeRenderData();
}

void CNeon::Scale(float Scale)
{
	int i;
	
	for(i=0;i<nVertexList;i++)
	{
		VertexList[i][0]*=Scale;
		VertexList[i][1]*=Scale;
	}
	
	for(i=0;i<nVertexList;i+=2)
	{
		NormalList[i/2][0] = VertexList[i+1][1] - VertexList[i][1];
		NormalList[i/2][1] = VertexList[i][0] - VertexList[i+1][0];
		
		vec2normalize(NormalList[i/2]);
		
		NormalList[i/2][2] = -(NormalList[i/2][0] * VertexList[i][0] + NormalList[i/2][1] * VertexList[i][1]);
	}
	
	MakeRenderData();
}

void CNeon::MakeRenderData()
{
	int i;
	int a, b, ni, ri=0;
	float scale=4.0f;
	for(i=0;i<nVertexList;i+=2)
	{
		a = i;
		b = i + 1;
		ni = i / 2;
		
		// p1
		rVL[ri][0] = VertexList[a][0] + (NormalList[ni][0]*scale);
		rVL[ri][1] = VertexList[a][1] + (NormalList[ni][1]*scale);
		rTL[ri][0] = 0.0f;
		rTL[ri][1] = 0.5f;
		ri++;
		
		// p2
		rVL[ri][0] = VertexList[a][0] - (NormalList[ni][0]*scale);
		rVL[ri][1] = VertexList[a][1] - (NormalList[ni][1]*scale);
		rTL[ri][0] = 1.0f;
		rTL[ri][1] = 0.5f;
		ri++;
		
		// p3
		rVL[ri][0] = VertexList[b][0] + (NormalList[ni][0]*scale);
		rVL[ri][1] = VertexList[b][1] + (NormalList[ni][1]*scale);
		rTL[ri][0] = 0.0f;
		rTL[ri][1] = 0.5f;
		ri++;
		
		// p2
		rVL[ri][0] = VertexList[a][0] - (NormalList[ni][0]*scale);
		rVL[ri][1] = VertexList[a][1] - (NormalList[ni][1]*scale);
		rTL[ri][0] = 1.0f;
		rTL[ri][1] = 0.5f;
		ri++;
		
		// p4
		rVL[ri][0] = VertexList[b][0] - (NormalList[ni][0]*scale);
		rVL[ri][1] = VertexList[b][1] - (NormalList[ni][1]*scale);
		rTL[ri][0] = 1.0f;
		rTL[ri][1] = 0.5f;
		ri++;
		
		// p3
		rVL[ri][0] = VertexList[b][0] + (NormalList[ni][0]*scale);
		rVL[ri][1] = VertexList[b][1] + (NormalList[ni][1]*scale);
		rTL[ri][0] = 0.0f;
		rTL[ri][1] = 0.5f;
		ri++;		
	}
}