#import "game_enemy.h"
#import "game_tower.h"
#import "game_common.h"
#import "me_Timer.h"

CEnemy g_Enemy;

void CEnemy::Init()
{
	m_EnemyTex[ET_SOLDIER]=me_CreateTexture(@"enemy_soldier.png");
	m_EnemyTex[ET_CAR]=me_CreateTexture(@"enemy_car.png");
	m_EnemyTex[ET_TANK]=me_CreateTexture(@"enemy_tank.png");
	
}

int CEnemy::Empty()
{
	int i;
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type != ET_NONE) return 0;
	}
	
	return 1;
}

void CEnemy::FrameMove()
{
	int i;
	vector2f d;
	
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type == ET_NONE) continue;
		if(m_Enemy[i].WaitTime > 0) 
		{
			m_Enemy[i].WaitTime -= me_GetElapsedTime();
			continue;
		}
		
		if(m_Enemy[i].HP<=0)
		{
			switch(m_Enemy[i].Type)
			{
				case ET_SOLDIER:g_Score += 5; g_Money += 1; break;
				case ET_CAR:	g_Score += 15; g_Money += 3; break;
				case ET_TANK:	g_Score += 30; g_Money += 5; break;
			}
			
			m_Enemy[i].Type = ET_NONE;
			continue;
		}
		
		if(m_Enemy[i].Pos[0] > m_Enemy[i].TargetPos[0])
		{
			m_Enemy[i].Pos[0] -= m_Enemy[i].Speed;
			m_Enemy[i].Angle = 270;
		}
			
		if(m_Enemy[i].Pos[0] < m_Enemy[i].TargetPos[0])
		{
			m_Enemy[i].Pos[0] += m_Enemy[i].Speed;
			m_Enemy[i].Angle = 90;
		}
			
		if(m_Enemy[i].Pos[1] > m_Enemy[i].TargetPos[1])
		{
			m_Enemy[i].Pos[1] -= m_Enemy[i].Speed;
			m_Enemy[i].Angle = 0;
		}
			
		if(m_Enemy[i].Pos[1] < m_Enemy[i].TargetPos[1])
		{
			m_Enemy[i].Pos[1] += m_Enemy[i].Speed;
			m_Enemy[i].Angle = 180;
		}
			
		vec2sub(d, m_Enemy[i].Pos, m_Enemy[i].TargetPos);
		if(vec2length(d)<m_Enemy[i].Speed)
		{
			m_Enemy[i].CurPathIndex--;
			if(m_Enemy[i].CurPathIndex<0) 
			{
				m_Enemy[i].Type = ET_NONE;
				continue;
			}
				
			m_Enemy[i].TargetPos[0] = m_Enemy[i].PathIndexX[m_Enemy[i].CurPathIndex] * 32.0f;
			m_Enemy[i].TargetPos[1] = m_Enemy[i].PathIndexY[m_Enemy[i].CurPathIndex] * 32.0f;
		}
	}
}

void CEnemy::Render()
{
	int i;
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type == ET_NONE) continue;
		
		me_PutSpr(m_Enemy[i].Pos[0], m_Enemy[i].Pos[1], 
				  m_EnemyTex[m_Enemy[i].Type],  m_Enemy[i].Angle);
	}
}

void CEnemy::FindPathAgain()
{
	int i;
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type == ET_NONE) continue;
		
		g_Tower.PathFind(m_Enemy[i].Pos[0]/32, m_Enemy[i].Pos[1]/32, 14, 4, m_Enemy[i].PathIndexX, m_Enemy[i].PathIndexY, m_Enemy[i].PathIndexCount);
		m_Enemy[i].CurPathIndex = m_Enemy[i].PathIndexCount-1;
	}
}

void CEnemy::Create(int Type, int HP, float Speed, float WaitTime)
{
	int i;
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type == ET_NONE)
		{
			m_Enemy[i].Type = Type;
			m_Enemy[i].HP = HP;
			m_Enemy[i].Speed = Speed;
			m_Enemy[i].WaitTime = WaitTime;
			
			m_Enemy[i].Pos[0]=-32.0f;
			m_Enemy[i].Pos[1]=128;
			
			m_Enemy[i].TargetPos[0]=-32.0f;
			m_Enemy[i].TargetPos[1]=128;
			
			g_Tower.PathFind(0, 4, 14, 4, m_Enemy[i].PathIndexX, m_Enemy[i].PathIndexY, m_Enemy[i].PathIndexCount);
			m_Enemy[i].CurPathIndex = m_Enemy[i].PathIndexCount-1;
			
			break;
		}
	}
}

int CEnemy::Search(vector2f Pos, float radius)
{
	int i;
	vector2f delta;
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type == ET_NONE) continue;
		if(m_Enemy[i].WaitTime > 0) continue;
		
		vec2sub(delta, m_Enemy[i].Pos, Pos);
		
		if(vec2length(delta)<=radius)
		{
			return i;
		}
	}
	
	return -1;
}

void CEnemy::SplashDamage(vector2f Pos, float radius, int Pow, int Except)
{
	int i;
	vector2f delta;
	for(i=0;i<MAX_ENEMY;i++)
	{
		if(m_Enemy[i].Type == ET_NONE) continue;
		if(m_Enemy[i].WaitTime > 0) continue;
		if(i == Except) continue;
		
		vec2sub(delta, m_Enemy[i].Pos, Pos);
		
		if(vec2length(delta)<=radius)
		{
			m_Enemy[i].HP -= Pow;
			//if(m_Enemy[i].HP <= 0) m_Enemy[i].Type = ET_NONE;
		}
	}
}