#import "game_tower.h"
#import "me_Timer.h"
#import "game_enemy.h"
#import "game_bullet.h"

CTower g_Tower;

void CTower::Init()
{
	m_TowerTex[TW_GUN]=me_CreateTexture(@"tower_gun.png");
	m_TowerTex[TW_MISSILE]=me_CreateTexture(@"tower_missile.png");
	m_TowerTex[TW_PLASMA]=me_CreateTexture(@"tower_plasma.png");
}

int CTower::Create(int x, int y, int Type)
{
	if(m_TowerMap[y][x].Type==TW_NONE)
	{
		m_TowerMap[y][x].Type=Type;
		
		m_TowerMap[y][x].CurFireDelay = 0;
		m_TowerMap[y][x].Angle = 0;
		m_TowerMap[y][x].TargetIndex = -1;
		
		switch(Type)
		{
			case TW_GUN:
				m_TowerMap[y][x].Pow = 1;
				m_TowerMap[y][x].FireDelay = 0.1f;
				m_TowerMap[y][x].Radius = 92;
				break;
			case TW_MISSILE:
				m_TowerMap[y][x].Pow = 10;
				m_TowerMap[y][x].FireDelay = 1.0f;
				m_TowerMap[y][x].Radius = 128;
				break;
			case TW_PLASMA:
				m_TowerMap[y][x].Pow = 2;
				m_TowerMap[y][x].FireDelay = 0.5f;
				m_TowerMap[y][x].Radius = 192;
				break;
		}
		
		return YES;
	}
	
	return NO;
}

void CTower::Delete(int x, int y)
{
	m_TowerMap[y][x].Type = TW_NONE;
}

void CTower::FrameMove()
{
	int i, j;
	vector2f Pos;
	vector2f delta;
	
	for(i=0;i<10;i++)
	{
		for(j=0;j<15;j++)
		{
			if(m_TowerMap[i][j].Type==TW_NONE) continue;
			
			Pos[0]=j * 32 + 16; 
			Pos[1]=i * 32 + 16;
			
			if(m_TowerMap[i][j].TargetIndex == -1)
			{
				m_TowerMap[i][j].TargetIndex = g_Enemy.Search(Pos, m_TowerMap[i][j].Radius);
			}
			else 
			{
				SEnemy *tEnemy = g_Enemy.GetEnemy(m_TowerMap[i][j].TargetIndex);
				if(tEnemy->Type == ET_NONE)
				{
					m_TowerMap[i][j].TargetIndex = -1;
					continue;
				}
				
				vec2sub(delta, Pos, tEnemy->Pos);
				if(vec2length(delta)>m_TowerMap[i][j].Radius)
				{
					m_TowerMap[i][j].TargetIndex = -1;
					continue;
				}
				
				m_TowerMap[i][j].Angle = GetAngle(Pos, tEnemy->Pos);
				
				m_TowerMap[i][j].CurFireDelay -= me_GetElapsedTime();
				if(m_TowerMap[i][j].CurFireDelay <= 0)
				{
					m_TowerMap[i][j].CurFireDelay = m_TowerMap[i][j].FireDelay;
					
					g_Bullet.Create(Pos[0], Pos[1],	m_TowerMap[i][j].Type, 
									m_TowerMap[i][j].TargetIndex, 
									m_TowerMap[i][j].Angle, m_TowerMap[i][j].Pow);
									
				}
			}
		}
	}
}

void CTower::Render()
{
	int i, j;
	
	for(i=0;i<10;i++)
	{
		for(j=0;j<15;j++)
		{
			if(m_TowerMap[i][j].Type==TW_NONE) continue;
			me_PutSpr(j*32, i*32, m_TowerTex[m_TowerMap[i][j].Type], m_TowerMap[i][j].Angle);
		}
	}
}

void CTower::FindAroundOpenNode(int CloseNodeX, int CloseNodeY)
{
	int i;
	int x, y;
	int G,H,F;
	
	for(i=0;i<4;i++)
	{
		switch(i)
		{
			case 0:
				x = CloseNodeX + 1; y = CloseNodeY; 
				break;
			case 1:
				x = CloseNodeX - 1; y = CloseNodeY; 
				break;
			case 2:
				x = CloseNodeX; y = CloseNodeY + 1; 
				break;
			case 3:
				x = CloseNodeX; y = CloseNodeY - 1; 
				break;
		}
		
		if(x<0) continue;
		if(x>=15) continue;
		if(y<0) continue;
		if(y>=10) continue;
		
		if(m_Node[y][x].Type == PF_CLOSE_NODE) continue;
		if(m_TowerMap[y][x].Type != TW_NONE) continue;
		
		G = m_Node[CloseNodeY][CloseNodeX].G + 1;
		H = sqrtf((m_EndPosX - CloseNodeX) * 
				  (m_EndPosX - CloseNodeX) + 
				  (m_EndPosY - CloseNodeY) * 
				  (m_EndPosY - CloseNodeY));
		
		F = G + H;
		
		if(m_Node[y][x].Type == PF_NONE)
		{
			m_Node[y][x].Type = PF_OPEN_NODE;
			m_Node[y][x].F = F;
			m_Node[y][x].G = G;
			m_Node[y][x].H = H;
			m_Node[y][x].ParentX = CloseNodeX;
			m_Node[y][x].ParentY = CloseNodeY;			
		}
		else 
		{
			if(m_Node[y][x].F > F)
			{
				m_Node[y][x].F = F;
				m_Node[y][x].G = G;
				m_Node[y][x].H = H;
				m_Node[y][x].ParentX = CloseNodeX;
				m_Node[y][x].ParentY = CloseNodeY;	
			}
		}				  
	}
}

int CTower::PathFindAndOpenNode2CloseNode()
{
	int lowestF = 0x0fffffff;
	int lowestX = -1, lowestY = -1;
	
	int i,j;
	for(i=0;i<10;i++)
	{
		for(j=0;j<15;j++)
		{
			if(m_Node[i][j].Type == PF_OPEN_NODE)
			{
				if(lowestF > m_Node[i][j].F)
				{
					lowestF = m_Node[i][j].F;
					lowestX = j;
					lowestY = i;
				}
			}
		}
	}
	
	if(lowestX==-1 && lowestY==-1) 
		return PF_RESULT_NOWAY;
	
	if(lowestX == m_EndPosX && lowestY == m_EndPosY)
		return PF_RESULT_OK;
	
	m_Node[lowestY][lowestX].Type = PF_CLOSE_NODE;
	
	FindAroundOpenNode(lowestX, lowestY);
	
	return PF_RESULT_NEXT;
}

int CTower::PathFind(int sx, int sy, int ex, int ey, int *PathIndexX, int *PathIndexY, int &PathIndexCount)
{
	m_StartPosX = sx;
	m_StartPosY = sy;
	m_EndPosX = ex;
	m_EndPosY = ey;
	
	int i, j;
	for(i=0;i<10;i++)
	{
		for(j=0;j<15;j++)
		{
			m_Node[i][j].Type = PF_NONE;
		}
	}
	
	m_Node[m_StartPosY][m_StartPosX].Type = PF_CLOSE_NODE;
	m_Node[m_StartPosY][m_StartPosX].F = 0;
	m_Node[m_StartPosY][m_StartPosX].G = 0;
	m_Node[m_StartPosY][m_StartPosX].H = 0;
	FindAroundOpenNode(m_StartPosX, m_StartPosY);
	
	int result;
	while(1)
	{
		result = PathFindAndOpenNode2CloseNode();
		
		if(result == PF_RESULT_OK) break;
		if(result == PF_RESULT_NOWAY) return NO;
	}
	
	PathIndexCount = 0;
	int x=m_EndPosX, y=m_EndPosY;
	int tx, ty;
	while(1)
	{
		PathIndexX[PathIndexCount]=x;
		PathIndexY[PathIndexCount]=y;
		
		PathIndexCount++;
		
		if( x == m_StartPosX && y == m_StartPosY) break;
		
		tx = m_Node[y][x].ParentX;
		ty = m_Node[y][x].ParentY;
		x = tx;
		y = ty;
	}	
	
	return YES;
}